import _extends from "@babel/runtime/helpers/extends";
import _objectWithoutProperties from "@babel/runtime/helpers/objectWithoutProperties";
const _excluded = ["root", "value", "children"];
import React from 'react';
import is from '@amaui/utils/is';
import { AmauiStyle, makeClassName, unit, rtl, sort, valueObject } from '@amaui/style';
import AmauiStyleContext from './AmauiStyleContext';

function makeAmauiStyle(element) {
  const amauiStyle = new AmauiStyle(element); // Add all the plugins

  amauiStyle.plugins.add = [unit, makeClassName, rtl, sort, valueObject];
  return amauiStyle;
}

const AmauiStyleProvider = /*#__PURE__*/React.forwardRef((props, ref) => {
  const {
    root = false,
    value: value_,
    children
  } = props,
        other = _objectWithoutProperties(props, _excluded);

  const refs = {
    root: React.useRef()
  };
  const [value, setValue] = React.useState(() => {
    if (value_ === undefined || !(value_ instanceof AmauiStyle)) return makeAmauiStyle();
    return value_;
  });
  React.useEffect(() => {
    if (refs.root.current) {
      value.element = refs.root.current; // Init

      value.init();
      const valueNew = new AmauiStyle(); // Copy over from value

      Object.keys(value).forEach(prop => valueNew[prop] = value[prop]);
      setValue(valueNew);
    }
  }, []);

  const update = updateValue => {
    if (updateValue !== undefined) {
      const valueNew = new AmauiStyle();
      Object.keys(value).forEach(prop => valueNew[prop] = value[prop]);
      is('object', updateValue) && Object.keys(updateValue).forEach(prop => valueNew[prop] = updateValue[prop]);
      setValue(valueNew);
      return valueNew;
    }
  }; // Update method


  value.updateWithRerender = update;
  if (root) return /*#__PURE__*/React.createElement(AmauiStyleContext.Provider, {
    value: value
  }, /*#__PURE__*/React.createElement("div", _extends({
    ref: item => {
      refs.root.current = item;
      if (ref !== null && ref !== void 0 && ref.current) ref.current = item;
    }
  }, other), children));
  return /*#__PURE__*/React.createElement(AmauiStyleContext.Provider, {
    value: value
  }, children);
});
export default AmauiStyleProvider;